@@ -1,7 +1,7 @@
 /***********************************************************
 * Rocks'n'Diamonds -- McDuffin Strikes Back!               *
 *----------------------------------------------------------*
-* (c) 1995-2002 Artsoft Entertainment                      *
+* (c) 1995-2006 Artsoft Entertainment                      *
 *               Holger Schemel                             *
 *               Detmolder Strasse 189                      *
 *               33604 Bielefeld                            *
@@ -345,102 +345,6 @@ void DrawVideoDisplay(unsigned long state, unsigned long value)
     },
   };
 
-#if 0
-  if (state & VIDEO_STATE_FFWD_OFF)
-  {
-#if 1
-    int cx = DOOR_GFX_PAGEX3, cy = DOOR_GFX_PAGEY2;
-
-    if (value != VIDEO_DISPLAY_SYMBOL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_LABEL_XPOS,
-		 cy + VIDEO_LABEL_YPOS,
-		 VIDEO_LABEL_XSIZE,
-		 VIDEO_LABEL_YSIZE,
-		 VX + VIDEO_LABEL_XPOS,
-		 VY + VIDEO_LABEL_YPOS);
-
-    cx = DOOR_GFX_PAGEX3;
-    cy = DOOR_GFX_PAGEY2;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_FFWD_SYMBOL_XPOS,
-		 cy + VIDEO_SYMBOL_YPOS,
-		 VIDEO_FFWD_SYMBOL_XSIZE,
-		 VIDEO_FFWD_SYMBOL_YSIZE,
-		 VX + VIDEO_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-#else
-    int cx = DOOR_GFX_PAGEX3, cy = DOOR_GFX_PAGEY2;
-
-    BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-	       cx + VIDEO_PLAY_SYMBOL_XPOS - 9,
-	       cy + VIDEO_PLAY_SYMBOL_YPOS,
-	       VIDEO_PLAY_SYMBOL_XSIZE - 2,
-	       VIDEO_PLAY_SYMBOL_YSIZE,
-	       VX + VIDEO_PLAY_SYMBOL_XPOS - 9,
-	       VY + VIDEO_PLAY_SYMBOL_YPOS);
-#endif
-  }
-#endif
-
-#if 0
-  if (state & VIDEO_STATE_PBEND_OFF)
-  {
-#if 1
-    int cx = DOOR_GFX_PAGEX3, cy = DOOR_GFX_PAGEY2;
-
-    if (value != VIDEO_DISPLAY_SYMBOL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_LABEL_XPOS,
-		 cy + VIDEO_LABEL_YPOS,
-		 VIDEO_LABEL_XSIZE,
-		 VIDEO_LABEL_YSIZE,
-		 VX + VIDEO_LABEL_XPOS,
-		 VY + VIDEO_LABEL_YPOS);
-
-    cx = DOOR_GFX_PAGEX3;
-    cy = DOOR_GFX_PAGEY2;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_PBEND_SYMBOL_XPOS,
-		 cy + VIDEO_SYMBOL_YPOS,
-		 VIDEO_PBEND_SYMBOL_XSIZE,
-		 VIDEO_PBEND_SYMBOL_YSIZE,
-		 VX + VIDEO_PBEND_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-#else
-    int cx = DOOR_GFX_PAGEX3, cy = DOOR_GFX_PAGEY2;
-
-    BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-	       cx + VIDEO_REC_LABEL_XPOS,
-	       cy + VIDEO_REC_LABEL_YPOS,
-	       VIDEO_PBEND_LABEL_XSIZE,
-	       VIDEO_PBEND_LABEL_YSIZE,
-	       VX + VIDEO_REC_LABEL_XPOS,
-	       VY + VIDEO_REC_LABEL_YPOS);
-#endif
-  }
-#endif
-
-#if 0
-  if (state & VIDEO_STATE_1STEP_OFF)
-  {
-    int cx = DOOR_GFX_PAGEX3, cy = DOOR_GFX_PAGEY2;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_1STEP_SYMBOL_XPOS,
-		 cy + VIDEO_SYMBOL_YPOS,
-		 VIDEO_1STEP_SYMBOL_XSIZE,
-		 VIDEO_1STEP_SYMBOL_YSIZE,
-		 VX + VIDEO_1STEP_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-  }
-#endif
-
   for (k = 0; k < NUM_TAPE_FUNCTION_STATES; k++)	/* on or off states */
   {
     for (i = 0; i < NUM_TAPE_FUNCTIONS; i++)		/* record, play, ... */
@@ -476,102 +380,6 @@ void DrawVideoDisplay(unsigned long state, unsigned long value)
     }
   }
 
-#if 0
-  if (state & VIDEO_STATE_FFWD_ON)
-  {
-#if 1
-    int cx = DOOR_GFX_PAGEX6, cy = DOOR_GFX_PAGEY1;
-
-    if (value != VIDEO_DISPLAY_SYMBOL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_FFWD_LABEL_XPOS,
-		 cy + VIDEO_FFWD_LABEL_YPOS,
-		 VIDEO_LABEL_XSIZE,
-		 VIDEO_LABEL_YSIZE,
-		 VX + VIDEO_LABEL_XPOS,
-		 VY + VIDEO_LABEL_YPOS);
-
-    cx = DOOR_GFX_PAGEX6;
-    cy = DOOR_GFX_PAGEY1;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_FFWD_SYMBOL_XPOS,
-		 cy + VIDEO_FFWD_SYMBOL_YPOS,
-		 VIDEO_FFWD_SYMBOL_XSIZE,
-		 VIDEO_FFWD_SYMBOL_YSIZE,
-		 VX + VIDEO_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-#else
-    int cx = DOOR_GFX_PAGEX4, cy = DOOR_GFX_PAGEY2;
-
-    BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-	       cx + VIDEO_PLAY_SYMBOL_XPOS,
-	       cy + VIDEO_PLAY_SYMBOL_YPOS,
-	       VIDEO_PLAY_SYMBOL_XSIZE - 2,
-	       VIDEO_PLAY_SYMBOL_YSIZE,
-	       VX + VIDEO_PLAY_SYMBOL_XPOS - 9,
-	       VY + VIDEO_PLAY_SYMBOL_YPOS);
-#endif
-  }
-#endif
-
-#if 0
-  if (state & VIDEO_STATE_PBEND_ON)
-  {
-#if 1
-    int cx = DOOR_GFX_PAGEX6, cy = DOOR_GFX_PAGEY1;
-
-    if (value != VIDEO_DISPLAY_SYMBOL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_PBEND_LABEL_XPOS,
-		 cy + VIDEO_PBEND_LABEL_YPOS,
-		 VIDEO_LABEL_XSIZE,
-		 VIDEO_LABEL_YSIZE,
-		 VX + VIDEO_LABEL_XPOS,
-		 VY + VIDEO_LABEL_YPOS);
-
-    cx = DOOR_GFX_PAGEX6;
-    cy = DOOR_GFX_PAGEY1;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_PBEND_SYMBOL_XPOS,
-		 cy + VIDEO_PBEND_SYMBOL_YPOS,
-		 VIDEO_PBEND_SYMBOL_XSIZE,
-		 VIDEO_PBEND_SYMBOL_YSIZE,
-		 VX + VIDEO_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-#else
-    int cx = DOOR_GFX_PAGEX6, cy = DOOR_GFX_PAGEY1;
-
-    BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-	       cx + VIDEO_PBEND_LABEL_XPOS,
-	       cy + VIDEO_PBEND_LABEL_YPOS,
-	       VIDEO_PBEND_LABEL_XSIZE,
-	       VIDEO_PBEND_LABEL_YSIZE,
-	       VX + VIDEO_REC_LABEL_XPOS,
-	       VY + VIDEO_REC_LABEL_YPOS);
-#endif
-  }
-#endif
-
-#if 0
-  if (state & VIDEO_STATE_1STEP_ON)
-  {
-    int cx = DOOR_GFX_PAGEX6, cy = DOOR_GFX_PAGEY1;
-
-    if (value != VIDEO_DISPLAY_LABEL_ONLY)
-      BlitBitmap(graphic_info[IMG_GLOBAL_DOOR].bitmap, drawto,
-		 cx + VIDEO_1STEP_SYMBOL_XPOS,
-		 cy + VIDEO_1STEP_SYMBOL_YPOS,
-		 VIDEO_1STEP_SYMBOL_XSIZE,
-		 VIDEO_1STEP_SYMBOL_YSIZE,
-		 VX + VIDEO_1STEP_SYMBOL_XPOS,
-		 VY + VIDEO_SYMBOL_YPOS);
-  }
-#endif
-
   if (state & VIDEO_STATE_DATE_ON)
   {
     int tag = value % 100;
@@ -597,16 +405,7 @@ void DrawVideoDisplay(unsigned long state, unsigned long value)
 	     int2str(sec, 2), FONT_TAPE_RECORDER);
   }
 
-#if 1
   redraw_mask |= REDRAW_DOOR_2;
-#else
-  if (state & VIDEO_STATE_DATE)
-    redraw_mask |= REDRAW_VIDEO_1;
-  if ((state & ~VIDEO_STATE_DATE) & VIDEO_STATE)
-    redraw_mask |= REDRAW_VIDEO_2;
-  if (state & VIDEO_PRESS)
-    redraw_mask |= REDRAW_VIDEO_3;
-#endif
 }
 
 void DrawCompleteVideoDisplay()
@@ -684,13 +483,11 @@ void TapeErase()
 
   TapeSetDate();
 
-#if 0
-  printf("::: tape.engine_version = level.game_version = %d \n",
-	 level.game_version);
-#endif
-
   for (i = 0; i < MAX_PLAYERS; i++)
     tape.player_participates[i] = FALSE;
+
+  tape.centered_player_nr_next = -1;
+  tape.set_centered_player = FALSE;
 }
 
 static void TapeRewind()
@@ -708,16 +505,25 @@ static void TapeRewind()
   tape.quick_resume = FALSE;
   tape.single_step = FALSE;
 
+  tape.centered_player_nr_next = -1;
+  tape.set_centered_player = FALSE;
+
   InitRND(tape.random_seed);
 }
 
-void TapeStartRecording()
+static void TapeSetRandomSeed(long random_seed)
+{
+  tape.random_seed = InitRND(random_seed);
+}
+
+void TapeStartRecording(long random_seed)
 {
   if (!TAPE_IS_STOPPED(tape))
     TapeStop();
 
   TapeErase();
   TapeRewind();
+  TapeSetRandomSeed(random_seed);
 
   tape.recording = TRUE;
 
@@ -732,18 +538,18 @@ void TapeStartRecording()
 
 static void TapeStartGameRecording()
 {
-  TapeStartRecording();
+  TapeStartRecording(NEW_RANDOMIZE);
 
 #if defined(NETWORK_AVALIABLE)
   if (options.network)
-    SendToServer_StartPlaying();
-  else
-#endif
   {
-    game_status = GAME_MODE_PLAYING;
-    StopAnimation();
-    InitGame();
+    SendToServer_StartPlaying();
+
+    return;
   }
+#endif
+
+  InitGame();
 }
 
 static void TapeAppendRecording()
@@ -776,11 +582,6 @@ void TapeHaltRecording()
 
 void TapeStopRecording()
 {
-#if 0
-  if (!tape.recording)
-    return;
-#endif
-
   TapeHaltRecording();
 
   tape.recording = FALSE;
@@ -790,17 +591,13 @@ void TapeStopRecording()
   MapTapeEjectButton();
 }
 
-void TapeRecordAction(byte action[MAX_PLAYERS])
+void TapeRecordAction(byte action_raw[MAX_PLAYERS])
 {
+  byte action[MAX_PLAYERS];
   int i;
 
-#if 1
-  if (!tape.recording)		/* record action even when tape is paused! */
+  if (!tape.recording)		/* (record action even when tape is paused) */
     return;
-#else
-  if (!tape.recording || tape.pausing)
-    return;
-#endif
 
   if (tape.counter >= MAX_TAPE_LEN - 1)
   {
@@ -808,9 +605,18 @@ void TapeRecordAction(byte action[MAX_PLAYERS])
     return;
   }
 
-#if 0
-  printf("::: %05d: recording action: %d\n", FrameCounter, action[0]);
-#endif
+  for (i = 0; i < MAX_PLAYERS; i++)
+    action[i] = action_raw[i];
+
+  if (tape.set_centered_player)
+  {
+    for (i = 0; i < MAX_PLAYERS; i++)
+      if (tape.centered_player_nr_next == i ||
+	  tape.centered_player_nr_next == -1)
+	action[i] |= KEY_SET_FOCUS;
+
+    tape.set_centered_player = FALSE;
+  }
 
   if (tape.pos[tape.counter].delay > 0)		/* already stored action */
   {
@@ -831,11 +637,6 @@ void TapeRecordAction(byte action[MAX_PLAYERS])
 
   if (tape.pos[tape.counter].delay == 0)	/* store new action */
   {
-
-#if 0
-    printf("::: %05d: new sequence\n", FrameCounter);
-#endif
-
     for (i = 0; i < MAX_PLAYERS; i++)
       tape.pos[tape.counter].action[i] = action[i];
 
@@ -847,11 +648,6 @@ void TapeTogglePause(boolean toggle_manual)
 {
   int state = 0;
 
-#if 0
-  if (!tape.recording && !tape.playing)
-    return;
-#endif
-
   if (tape.pause_before_death)
     state |= VIDEO_STATE_PBEND_OFF;
   else if (tape.fast_forward)
@@ -872,11 +668,6 @@ void TapeTogglePause(boolean toggle_manual)
 
   DrawVideoDisplay(state, 0);
 
-#if 0
-  if (tape.pausing)
-    DrawVideoDisplay(VIDEO_STATE_PAUSE_ON, 0);
-#endif
-
   if (tape.warp_forward)
   {
     TapeStopWarpForward();
@@ -886,9 +677,9 @@ void TapeTogglePause(boolean toggle_manual)
       tape.quick_resume = FALSE;
 
       TapeAppendRecording();
-#if 0
-      TapeTogglePause(toggle_manual);
-#endif
+
+      if (!CheckEngineSnapshot())
+	SaveEngineSnapshot();
     }
   }
 }
@@ -918,18 +709,11 @@ static void TapeStartGamePlaying()
 {
   TapeStartPlaying();
 
-  game_status = GAME_MODE_PLAYING;
-  StopAnimation();
   InitGame();
 }
 
 void TapeStopPlaying()
 {
-#if 0
-  if (!tape.playing)
-    return;
-#endif
-
   tape.playing = FALSE;
   tape.pausing = FALSE;
 
@@ -1015,6 +799,21 @@ byte *TapePlayAction()
   for (i = 0; i < MAX_PLAYERS; i++)
     action[i] = tape.pos[tape.counter].action[i];
 
+  tape.set_centered_player = FALSE;
+  tape.centered_player_nr_next = -999;
+
+  for (i = 0; i < MAX_PLAYERS; i++)
+  {
+    if (action[i] & KEY_SET_FOCUS)
+    {
+      tape.set_centered_player = TRUE;
+      tape.centered_player_nr_next =
+	(tape.centered_player_nr_next == -999 ? i : -1);
+    }
+
+    action[i] &= ~KEY_SET_FOCUS;
+  }
+
   tape.delay_played++;
   if (tape.delay_played >= tape.pos[tape.counter].delay)
   {
@@ -1022,22 +821,11 @@ byte *TapePlayAction()
     tape.delay_played = 0;
   }
 
-#if 0
-  printf("::: %05d: replaying action: %d\n", FrameCounter, action[0]);
-#endif
-
   return action;
 }
 
 void TapeStop()
 {
-#if 0
-  if (tape.recording)
-    printf("::: stopped recording: %d\n", FrameCounter);
-  else if (tape.playing)
-    printf("::: stopped playing:   %d\n\n", FrameCounter);
-#endif
-
   TapeStopRecording();
   TapeStopPlaying();
 
@@ -1048,11 +836,6 @@ void TapeStop()
     DrawVideoDisplay(VIDEO_STATE_DATE_ON, tape.date);
     DrawVideoDisplay(VIDEO_STATE_TIME_ON, tape.length_seconds);
   }
-
-#if 0
-  if (tape.auto_play)
-    AutoPlayTape();	/* continue automatically playing next tape */
-#endif
 }
 
 unsigned int GetTapeLength()
@@ -1073,7 +856,6 @@ static void TapeStartWarpForward()
 {
   tape.warp_forward = TRUE;
 
-#if 1
   if (!tape.fast_forward && !tape.pause_before_death)
   {
     tape.pausing = FALSE;
@@ -1081,15 +863,6 @@ static void TapeStartWarpForward()
 
     TapeDeactivateDisplayOn();
   }
-#else
-  if (!tape.fast_forward || tape.pause_before_death)
-  {
-    tape.pausing = FALSE;
-    tape.deactivate_display = TRUE;
-
-    TapeDeactivateDisplayOn();
-  }
-#endif
 
   if (tape.fast_forward || tape.pause_before_death)
     DrawVideoDisplay(VIDEO_STATE_WARP_ON, VIDEO_DISPLAY_SYMBOL_ONLY);
@@ -1106,16 +879,6 @@ static void TapeStopWarpForward()
 
   TapeDeactivateDisplayOff(game_status == GAME_MODE_PLAYING);
 
-#if 0
-#if 1
-  if (game_status == GAME_MODE_PLAYING)
-#endif
-  {
-    RedrawPlayfield(TRUE, 0,0,0,0);
-    DrawGameDoorValues();
-  }
-#endif
-
   if (tape.pause_before_death)
     state |= VIDEO_STATE_WARP_OFF | VIDEO_STATE_PBEND_ON;
   else if (tape.fast_forward)
@@ -1142,18 +905,25 @@ static void TapeSingleStep()
 
 void TapeQuickSave()
 {
-  if (game_status == GAME_MODE_PLAYING)
+  if (game_status == GAME_MODE_MAIN)
   {
-    if (tape.recording)
-      TapeHaltRecording();	/* prepare tape for saving on-the-fly */
-
-    if (TAPE_IS_EMPTY(tape))
-      Request("No tape that can be saved !", REQ_CONFIRM);
-    else
-      SaveTape(tape.level_nr);
-  }
-  else if (game_status == GAME_MODE_MAIN)
     Request("No game that can be saved !", REQ_CONFIRM);
+
+    return;
+  }
+
+  if (game_status != GAME_MODE_PLAYING)
+    return;
+
+  if (tape.recording)
+    TapeHaltRecording();	/* prepare tape for saving on-the-fly */
+
+  if (TAPE_IS_EMPTY(tape))
+    Request("No tape that can be saved !", REQ_CONFIRM);
+  else
+    SaveTape(tape.level_nr);
+
+  SaveEngineSnapshot();
 }
 
 void TapeQuickLoad()
@@ -1175,25 +945,42 @@ void TapeQuickLoad()
     return;
   }
 
-  if (game_status == GAME_MODE_PLAYING || game_status == GAME_MODE_MAIN)
+  if (game_status != GAME_MODE_PLAYING && game_status != GAME_MODE_MAIN)
+    return;
+
+  if (CheckEngineSnapshot())
   {
-    TapeStop();
-    TapeErase();
+    TapeStartGamePlaying();
 
-    LoadTape(level_nr);
-    if (!TAPE_IS_EMPTY(tape))
-    {
-      TapeStartGamePlaying();
-      TapeStartWarpForward();
+    LoadEngineSnapshot();
 
-      tape.quick_resume = TRUE;
-    }
-    else	/* this should not happen (basically checked above) */
-    {
-      int reopen_door = (game_status == GAME_MODE_PLAYING ? REQ_REOPEN : 0);
+    tape.playing = TRUE;
+    tape.pausing = TRUE;
 
-      Request("No tape for this level !", REQ_CONFIRM | reopen_door);
-    }
+    TapeStopWarpForward();
+    TapeAppendRecording();
+
+    if (FrameCounter > 0)
+      return;
+  }
+
+  TapeStop();
+  TapeErase();
+
+  LoadTape(level_nr);
+
+  if (!TAPE_IS_EMPTY(tape))
+  {
+    TapeStartGamePlaying();
+    TapeStartWarpForward();
+
+    tape.quick_resume = TRUE;
+  }
+  else	/* this should not happen (basically checked above) */
+  {
+    int reopen_door = (game_status == GAME_MODE_PLAYING ? REQ_REOPEN : 0);
+
+    Request("No tape for this level !", REQ_CONFIRM | reopen_door);
   }
 }
 
@@ -1297,12 +1084,15 @@ void AutoPlayTape()
       continue;
     }
 
-    LoadSolutionTape(level_nr);
-#if 1
-    if (tape.no_valid_file)
-#else
-    if (TAPE_IS_EMPTY(tape))
+#if 0
+    /* ACTIVATE THIS FOR LOADING/TESTING OF LEVELS ONLY */
+    printf("(only testing level)\n");
+    continue;
 #endif
+
+    LoadSolutionTape(level_nr);
+
+    if (tape.no_valid_file)
     {
       num_tape_missing++;
       if (level_nr >= 0 && level_nr < MAX_TAPES_PER_SET)
@@ -1531,15 +1321,6 @@ static void HandleTapeButtons(struct GadgetInfo *gi)
 	else					/* WARP FORWARD PLAY -> PLAY */
 	{
 	  TapeStopWarpForward();
-
-#if 0
-	  if (tape.pause_before_death)
-	    DrawVideoDisplay(VIDEO_STATE_WARP_OFF | VIDEO_STATE_PLAY_ON, 0);
-	  else if (tape.fast_forward)
-	    DrawVideoDisplay(VIDEO_STATE_WARP_OFF | VIDEO_STATE_FFWD_ON, 0);
-	  else
-	    DrawVideoDisplay(VIDEO_STATE_WARP_OFF | VIDEO_STATE_PBEND_ON, 0);
-#endif
 	}
       }
       else if (tape.recording)
@@ -1588,28 +1369,17 @@ static void HandleTapeButtons(struct GadgetInfo *gi)
 	else if (!tape.pause_before_death)	/* FFWD PLAY -> AUTO PAUSE */
 	{
 	  tape.pause_before_death = TRUE;
-#if 1
 	  DrawVideoDisplay(VIDEO_STATE_FFWD_OFF | VIDEO_STATE_PBEND_ON, 0);
-#else
-	  DrawVideoDisplay(VIDEO_STATE_PBEND_ON, VIDEO_DISPLAY_LABEL_ONLY);
-#endif
 	}
 	else					/* AUTO PAUSE -> NORMAL PLAY */
 	{
-#if 1
 	  if (tape.warp_forward)
 	    TapeStopWarpForward();
-#else
-	  tape.warp_forward = FALSE;
-#endif
+
 	  tape.fast_forward = FALSE;
 	  tape.pause_before_death = FALSE;
 
-#if 1
 	  DrawVideoDisplay(VIDEO_STATE_PBEND_OFF | VIDEO_STATE_PLAY_ON, 0);
-#else
-	  DrawVideoDisplay(VIDEO_STATE_FFWD_OFF | VIDEO_STATE_PBEND_OFF, 0);
-#endif
 	}
       }
       break;
